@@ -1,10 +1,14 @@
 # This file is a part of the AnyBlok / Marshmallow api project
 #
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
 
-from .schema import ModelSchema  # noqa
+from .schema import ModelSchema, PostLoadSchema  # noqa
 from .exceptions import RegistryNotFound  # noqa
+from .fields import (  # noqa
+    Nested, File, Text, JsonCollection, PhoneNumber, Country, InstanceField
+)
