@@ -1,12 +1,14 @@
 # This file is a part of the AnyBlok / Marshmallow api project
 #
 #    Copyright (C) 2017 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
+#    Copyright (C) 2018 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
-from marshmallow import Schema, post_load, SchemaOpts, validates_schema
-from marshmallow.fields import Raw
+from marshmallow import (
+    Schema, post_load, SchemaOpts, validates_schema, validate
+)
 from marshmallow_sqlalchemy.schema import (
     ModelSchema as MS,
     ModelSchemaOpts as MSO
@@ -15,6 +17,10 @@ from marshmallow_sqlalchemy.convert import ModelConverter as MC
 from anyblok.common import anyblok_column_prefix
 from marshmallow.exceptions import ValidationError
 from .exceptions import RegistryNotFound
+from .fields import Raw, Nested, Text, Email, URL, PhoneNumber, Country
+import anyblok
+import sqlalchemy as sa
+import sqlalchemy_utils.types as sau
 
 
 def update_from_kwargs(*entries):
@@ -59,14 +65,69 @@ class ModelConverter(MC):
 
     The goal if to fix the fieldname, because they are prefixed.
     """
+    SQLA_TYPE_MAPPING = MC.SQLA_TYPE_MAPPING.copy()
+    SQLA_TYPE_MAPPING.update({
+        sa.Text: Text,
+        sau.email.EmailType: Email,
+        sau.url.URLType: URL,
+        sau.phone_number.PhoneNumberType: PhoneNumber,
+        anyblok.column.CountryType: Country
+    })
 
-    def fields_for_model(self, model, **kwargs):
+    def fields_for_model(self, Model, **kwargs):
         """Overwrite the method and remove prefix of the field name"""
-        res = super(ModelConverter, self).fields_for_model(model, **kwargs)
-        for field in model.loaded_fields.keys():
+        res = super(ModelConverter, self).fields_for_model(Model, **kwargs)
+        for field in Model.loaded_fields.keys():
             res[field] = Raw()
 
-        return {format_fields(x): y for x, y in res.items()}
+        fields = {format_fields(x): y for x, y in res.items()}
+        fields_description = Model.fields_description()
+        for field in fields:
+            if field not in fields_description:
+                continue
+
+            type_ = fields_description[field]['type']
+            if type_ == 'Selection':
+                validators = fields[field].validate
+                choices = list(dict(
+                    fields_description[field]['selections']).keys())
+                labels = list(dict(
+                    fields_description[field]['selections']).values())
+                validators.append(validate.OneOf(choices, labels=labels))
+            elif type_ in ('Many2One', 'One2One', 'One2Many', 'Many2Many'):
+                many = False if type_ in ('Many2One', 'One2One') else True
+                remote_model = fields_description[field]['model']
+                RemoteModel = Model.registry.get(remote_model)
+
+                sch = type(
+                    'Model.Schema.' + remote_model,
+                    (ModelSchema,),
+                    {'Meta': type('Meta', tuple(), {'model': remote_model})}
+                )
+
+                fields[field] = Nested(
+                    sch, many=many, only=RemoteModel.get_primary_keys())
+
+        return fields
+
+    def _add_column_kwargs(self, kwargs, column):
+        super(ModelConverter, self)._add_column_kwargs(kwargs, column)
+        required_fields = self.schema_cls.Meta.required_fields
+        if (
+            (isinstance(required_fields, (tuple, list)) and
+             column.name in required_fields) or
+            required_fields is True
+        ):
+            kwargs['required'] = True
+
+        if isinstance(column.type, sau.phone_number.PhoneNumberType):
+            kwargs['region'] = column.type.region
+        elif isinstance(column.type, anyblok.column.CountryType):
+            import pycountry
+            choices = [x.alpha_3 for x in pycountry.countries]
+            labels = [x.name for x in pycountry.countries]
+            validators = kwargs.get('validate', [])
+            validators.append(validate.OneOf(choices, labels=labels))
 
 
 class ModelSchemaOpts(SchemaOpts):
@@ -76,15 +137,13 @@ class ModelSchemaOpts(SchemaOpts):
 
     * model: name of an AnyBlok model **required**
     * registry: an AnyBlok registry
-    * post_load_return_instance: return an instance object
     """
     def __init__(self, meta, *args, **kwargs):
         super(ModelSchemaOpts, self).__init__(meta, *args, **kwargs)
         self.model = getattr(meta, 'model', False)
         self.registry = getattr(meta, 'registry', False)
-        self.post_load_return_instance = getattr(
-            meta, 'post_load_return_instance', False)
         self.only_primary_key = getattr(meta, 'only_primary_key', False)
+        self.required_fields = getattr(meta, 'required_fields', None)
 
 
 class TemplateSchema:
@@ -94,6 +153,34 @@ class TemplateSchema:
     dump = MS.dump
     validate = MS.validate
 
+    @validates_schema(pass_original=True)
+    def check_unknown_fields(self, data, original_data):
+        od = set(original_data.keys())
+        unknown = od - set(self.fields)
+        if unknown:
+            raise ValidationError(
+                'Unknown fields %r on Model %s' % (
+                    unknown, self.opts.model.__registry_name__
+                ),
+                unknown
+            )
+
+    @post_load
+    def make_instance(self, data):
+        return self.get_instance_from(data)
+
+    def get_instance_from(self, data):
+        try:
+            return super(TemplateSchema, self).get_instance_from(data)
+        except ValidationError:
+            raise
+        except Exception:
+            return data
+
+
+class PostLoadSchema:
+    post_load_attributes = True
+
     def valid_postload(self, postload, data, fields):
         if not postload:
             raise ValidationError(
@@ -124,44 +211,36 @@ class TemplateSchema:
 
         return postload
 
-    @post_load
-    def make_instance(self, data):
-        if self.post_load_return_instance is True:
+    def get_instance_from(self, data):
+        if self.post_load_attributes is True:
             Model = self.opts.model
             pks = Model.get_primary_keys()
             _pks = {x: data[x] for x in pks}
             return self.valid_postload(
                 Model.from_primary_keys(**_pks), data, pks)
 
-        if isinstance(self.post_load_return_instance, list):
-            # TODO filter add multi level
+        if isinstance(self.post_load_attributes, list):
             Model = self.opts.model
             query = Model.query()
             filter_by = {}
-            for field in self.post_load_return_instance:
+            for field in self.post_load_attributes:
+                if field not in data:
+                    raise ValidationError(
+                        {
+                            "KeyError": "%r is unknow in the data" % field,
+                        },
+                        fields_name=[field],
+                        data=data
+                    )
+
                 filter_by[field] = data[field]
 
             query = query.filter_by(**filter_by)
             return self.valid_postload(
-                query.all(), data, self.post_load_return_instance)
+                query.all(), data, self.post_load_attributes)
 
         return data
 
-    @validates_schema(pass_original=True)
-    def check_unknown_fields(self, data, original_data):
-        od = set()
-        for x in (original_data if self.many else [original_data]):
-            od.update(set(x))
-
-        unknown = od - set(self.fields)
-        if unknown:
-            raise ValidationError(
-                'Unknown fields %r on Model %s' % (
-                    unknown, self.opts.model.__registry_name__
-                ),
-                unknown
-            )
-
 
 class ModelSchema(Schema):
     """A marshmallow schema based on the AnyBlok Model
@@ -173,18 +252,17 @@ class ModelSchema(Schema):
 
     def __init__(self, *args, **kwargs):
         registry = kwargs.pop('registry', None)
-        post_load_return_instance = kwargs.pop(
-            'post_load_return_instance', None)
         only_primary_key = kwargs.pop('only_primary_key', None)
         model = kwargs.pop('model', None)
+        required_fields = kwargs.pop('required_fields', None)
+        self.instances = kwargs.pop('instances', {})
         super(ModelSchema, self).__init__(*args, **kwargs)
         self.args = args
         self.kwargs = kwargs
         self.registry = registry or self.opts.registry
         self.model = model or self.opts.model
+        self.required_fields = required_fields or self.opts.required_fields
         self.only_primary_key = only_primary_key or self.opts.only_primary_key
-        self.post_load_return_instance = (
-            post_load_return_instance or self.opts.post_load_return_instance)
 
     def get_registry(self):
         registry = self.context.get('registry', self.registry)
@@ -200,24 +278,21 @@ class ModelSchema(Schema):
     def get_model(self):
         return self.context.get('model', self.model)
 
-    def get_post_load_return_instance(self):
-        if self.post_load_return_instance:
-            return self.post_load_return_instance
-
-        return self.context.get('post_load_return_instance', False)
+    def get_required_fields(self):
+        return self.context.get('required_fields', self.required_fields)
 
     def generate_marsmallow_instance(self):
         """Generate the real mashmallow-sqlalchemy schema"""
         model = self.get_model()
         registry = self.get_registry()
-        post_load_return_instance = self.get_post_load_return_instance()
+        instances = self.context.get('instances', self.instances)
         only_primary_key = self.get_only_primary_key()
+        required_fields = self.get_required_fields()
 
         Schema = type(
             'Model.Schema.%s' % model,
             (TemplateSchema, self.__class__, MS),
             {
-                'post_load_return_instance': post_load_return_instance,
                 'Meta': type(
                     'Meta',
                     tuple(),
@@ -225,6 +300,7 @@ class ModelSchema(Schema):
                         'model': registry.get(model),
                         'sqla_session': registry.Session,
                         'model_converter': ModelConverter,
+                        'required_fields': required_fields,
                     },
                 ),
             }
@@ -239,7 +315,7 @@ class ModelSchema(Schema):
 
         schema = Schema(*self.args, **kwargs)
         schema.context['registry'] = registry
-        schema.context['post_load_return_instance'] = post_load_return_instance
+        schema.context['instances'] = instances
 
         return schema
 
@@ -248,18 +324,19 @@ class ModelSchema(Schema):
         """property to get the real schema"""
         return self.generate_marsmallow_instance()
 
-    @update_from_kwargs('registry', 'post_load_return_instance',
-                        'only_primary_key', 'model')
+    @update_from_kwargs('registry', 'only_primary_key', 'model', 'instances',
+                        'required_fields')
     def load(self, *args, **kwargs):
         """overload the main method to call in it in the real schema"""
         return self.schema.load(*args, **kwargs)
 
-    @update_from_kwargs('registry', 'only_primary_key', 'model')
+    @update_from_kwargs('registry', 'only_primary_key', 'model', 'instances')
     def dump(self, *args, **kwargs):
         """overload the main method to call in it in the real schema"""
         return self.schema.dump(*args, **kwargs)
 
-    @update_from_kwargs('registry', 'only_primary_key', 'model')
+    @update_from_kwargs('registry', 'only_primary_key', 'model', 'instances',
+                        'required_fields')
     def validate(self, *args, **kwargs):
         """overload the main method to call in it in the real schema"""
         return self.schema.validate(*args, **kwargs)
